<?php
	// JRankResultsParser.php
	require_once('JRankResult.php');

	/** Parses search results from a JRank query (see www.jrank.org)
	 *
	 * @author Hans de Ruiter
	 *
	 * @copyright (C) 2009 by Hans de Ruiter, all rights reserved (see license.txt for
	 * licensing information)
	 *
	 * @version 0.1 - 2009 March 12
	 */
	class JRankResultsParser extends XML
	{
		protected $searchURLBase;
		protected $metadata = array();
		protected $pages = array();
		protected $results = array();
		protected $inMetaSection = FALSE;
		protected $inPagesSection = FALSE;
		protected $inEntriesSection = FALSE;
		protected $currPage = NULL;
		protected $currResult = NULL;
		
		function __construct($searchURLBase)
		{
			parent::__construct();
			$this->searchURLBase = $searchURLBase;
		}
		
		/** Returns an array containing the meta-data
		 */
		public function getMetaData()
		{
			return new ArrayData($this->metadata);
		}
		
		public function getPages()
		{
			return new DataObjectSet($this->pages);
		}
		
		/** Returns an array containing the search results
		 */
		public function getResults()
		{
			return new DataObjectSet($this->results);
		}
		
		
		// ----- Parsing Functions -----
		function process_serp_end($collatedText, $attributes)
		{
			// Closing tag of the complete XML block
			// A quick sanity check
			if(array_key_exists('query', $this->metadata))
			{
				$this->parsed = TRUE;
			}
		}
		
		function process_meta($attributes){
			// Setting a flag to indicate that we're in the meta section
			$this->inMetaSection = TRUE;
		}
		
		function process_meta_end($collatedText, $attributes){
			// Clearing a flag to inficate that we're no longer the meta section
			$this->inMetaSection = FALSE;
		}
		
		function process_pages($attributes){
			// Setting a flag to indicate that we're in the pages section
			$this->inPagesSection = TRUE;
		}
		
		function process_pages_end($collatedText, $attributes){
			// Clearing a flag to inficate that we're no longer the pages section
			$this->inPagesSection = FALSE;
		}
		
		function process_entries($attributes){
			// Setting a flag to indicate that we're in the entries section
			$this->inEntriesSection = TRUE;
		}
		
		function process_entries_end($collatedText, $attributes)
		{
			// End of the pages block
			// Calculate a few remaining items
			$numPages = count($this->results);
			$this->metadata['end'] = $this->metadata['start'] + $numPages - 1;			
			
			// Exiting the entries section
			$this->inEntriesSection = FALSE;
		}
		
		function process_total_found_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			$this->metadata['total_found'] = (int)$collatedText;
		}
	
		function process_total_pages_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			$this->metadata['total_pages'] = (int)$collatedText;
		}
		
		function process_total_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			$this->metadata['total'] = (int)$collatedText;
		}
		
		function process_limit_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			$this->metadata['limit'] = $collatedText;
		}
		
		function process_time_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			$this->metadata['time'] = (int)$collatedText;
		}
		
		function process_query_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			$this->metadata['query'] = $collatedText;
		}
		
		function process_start_end($collatedText, $attributes)
		{ 
			if($this->inMetaSection == TRUE){
				$this->metadata['start'] = $collatedText + 1;
			}
			else if($this->inPagesSection == TRUE){
				$this->currPage['start'] = (int)$collatedText;
				$this->currPage['url'] = $this->searchURLBase."&start=".$collatedText;
			}
		}
		
		function process_next_page_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			if(strlen($collatedText) != 0)
			{
				$this->metadata['next_page'] = TRUE;
			}
		}
		
		function process_next_page_start_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			if(strlen($collatedText) != 0)
			{
				$this->metadata['next_page_start'] = $collatedText;
			}
		}
		
		function process_current_page_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			$this->metadata['current_page'] = $collatedText;
		}
		
		function process_prev_page_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			if(strlen($collatedText) != 0)
			{
				$this->metadata['prev_page'] = TRUE;
			}
		}
		
		function process_prev_page_start_end($collatedText, $attributes)
		{
			if(!$this->inMetaSection){
				return;
			}
			if(strlen($collatedText) != 0)
			{
				$this->metadata['prev_page_start'] = $collatedText;
			}
		}
		
		function process_page($attributes){
			if($this->inPagesSection == FALSE){
				return;
			}
			
			$this->currPage = array();
		}
		
		function process_page_end($collatedText, $attributes)
		{
			if($this->inPagesSection == FALSE){
				return;
			}
			// Add the completed entry to the list of results
			$pageObject = new ArrayData($this->currPage);
			array_push($this->pages, $pageObject);
			$this->currResult = NULL;
		}
		
		function process_number_end($collatedText, $attributes)
		{
			if($this->inPagesSection == FALSE){
				return;
			}
			$this->currPage['number'] = $collatedText;
			if($this->currPage['number'] == $this->metadata['current_page'])
			{
				$this->currPage['current_page'] = TRUE;
			}
		}
		
		function process_entry($attributes){
			if($this->inEntriesSection == FALSE){
				return;
			}
			
			$this->currResult = new JRankResult();
		}
		
		function process_entry_end($collatedText, $attributes)
		{
			if($this->inEntriesSection == FALSE){
				return;
			}
			// Add the completed entry to the list of results
			array_push($this->results, $this->currResult);
			$this->currResult = NULL;
		}
		
		function process_url_end($collatedText, $attributes)
		{
			if($this->currResult != NULL)
			{
				$this->currResult->setURL($collatedText);
			}
		}
		
		function process_title_end($collatedText, $attributes)
		{
			if($this->currResult != NULL)
			{
				$this->currResult->setTitle($collatedText);
			}
		}
		
		function process_content_end($collatedText, $attributes)
		{
			if($this->currResult != NULL)
			{
				$this->currResult->setContent($collatedText);
			}
		}
	}
 
 ?>